#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "conditionproxy4xi.py">
#<abstract4xi abstract = "provides class ConditionProxy4Xi"/>
#<refer4xi reference = "managers.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------


# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'ConditionProxy4Xi' ]
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass    
try:                 # most-common ones
    import weakref
    import array
    import queue
    from traceback import format_exc         
except:
    pass
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.managers import ConditionProxy
except:
    print("conditionproxy4xi.py *** failed in from multiprocessing.managers import ConditionProxy ***")
#"
#/>
     
# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from acquirerproxy4xi import AcquirerProxy4Xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from timeout4xi import omega2timeout4xi
from print4xi import print4xi
from time4xi import time4xi    
#"
#/>

#
# =================================================================================================
#<class4xi class = "ConditionProxy4Xi" base = "AcquirerProxy4Xi" child = "None" synopsis = "given as a parameter to SyncManager4Xi" usage = "
#
# usage
#     managers.py
#         SyncManager.register('Condition', threading.Condition, ConditionProxy)
#     syncmanager4xi.py:     
#         SyncManager4Xi.register('Condition4Xi', threading.Condition, ConditionProxy4Xi)"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
#
class ConditionProxy4Xi(AcquirerProxy4Xi):
    '''
    ConditionProxy4Xi class is given as a parameter to SyncManager4Xi
    '''

    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('wait', 'notify', 'notify_all', 'sense', 'reason', 'ace')

    #"
    #/>
                                 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait-of-ConditionProxy4Xi" usage = "returns with (self, predicate, timeout=None)">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def wait_for(self, predicate, timeout=None):
        '''
        ConditionProxy4Xi method wait: returns with (self, predicate, timeout=None)
        '''
        # ---------------------------------------------------------------------------------------------
        # (Step 1/3)
        # ---------------------------------------------------------------------------------------------
        class2print = 'ConditionProxy4Xi'
        method2print = "wait"
        
        message2print = "{(Step 1/3a) started" + "; wait-of-ConditionProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        
        message2print = "{(Step 1/3b) passed; predicate-passed-to-wait-for-of-ConditionProxy4Xi: " + str(predicate) + "}"
        print4xi(class2print, method2print, message2print)
        
        message2print = "{(Step 1/3c) passed; timeout-passed-to-wait-for-of-ConditionProxy4Xi: " + str(timeout) + "}"
        print4xi(class2print, method2print, message2print)    
                        
        timeout = omega2timeout4xi(class2print, method2print) 

        message2print = "{(Step 1/3d) set" + "; timeout-set-by-wait-of-ConditionProxy4Xi to: " + str(timeout)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # ---------------------------------------------------------------------------------------------
        # (Step 2/3)
        # ---------------------------------------------------------------------------------------------        
        result2xi = predicate()
        if result2xi:
            message2print = "{(Step 2/3a) returning" + "; result2xi-returning-by-wait-of-ConditionProxy4Xi: " + str(result2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)
            return result2xi
        if timeout is not None:
            endtime2xi = _time() + timeout
            message2print = "{(Step 2/3b) set" + "; endtime2xi-set-by-wait-of-ConditionProxy4Xi to: " + str(endtime2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
        else:
            endtime2xi = None
            waittime2xi = None
        while not result2xi:
            if endtime2xi is not None:
                waittime2xi = endtime2xi - _time()
                message2print = "{(Step 2/3c) set" + "; waittime2xi-set-by-wait-of-ConditionProxy4Xi to: " + str(waittime2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                if waittime2xi <= 0:
                    message2print = "{(Step 2/3d) got" + "; waittime2xi-got-by-wait-of-ConditionProxy4Xi: " + str(waittime2xi)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    break
            self.wait(waittime2xi)
            
            result2xi = predicate()
            message2print = "{(Step 2/3e) got" + "; result2xi-got-by-wait-of-ConditionProxy4Xi: " + str(result2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
                
        # ---------------------------------------------------------------------------------------------
        # (Step 3/3)
        # ---------------------------------------------------------------------------------------------        
        message2print = "{(Step 3/3) returning" + "; result2xi-returning-by-wait-of-ConditionProxy4Xi: " + str(result2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                        
        return result2xi
    #</methodize4xi>
            
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "notify-of-ConditionProxy4Xi" usage = "returns with (self)">                              
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def notify(self):
        '''
        ConditionProxy4Xi method notify: returns with (self)
        '''
        class2print = 'ConditionProxy4Xi'
        method2print = "notify"
        
        
        message2print = "{started" + "; notify-of-ConditionProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
     
        message2print = "{returning" + "; callmethod-of-conditionproxy2xi-returning-by-notify-of-ConditionProxy4Xi: " + str(self._callmethod) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                      
        
        return self._callmethod('notify')
    #</methodize4xi>
            
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "notify-all-of-ConditionProxy4Xi" usage = "returns  with (self)">                             
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def notify_all(self):
        '''
        ConditionProxy4Xi method notify_all: returns  with (self) 
        '''
        class2print = 'ConditionProxy4Xi'
        method2print = "notify_all"
        
        
        message2print = "{started" + "; notify-all-of-ConditionProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
     
     
        message2print = "{returning" + "; callmethod-of-conditionproxy2xi-returning-by-notify-all-of-ConditionProxy4Xi: " + str(self._callmethod) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                 
      
              
        return self._callmethod('notify_all')
    #</methodize4xi>
            
                                       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-ConditionProxy4Xi" usage = "senses ConditionProxy4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses ConditionProxy4Xi class objects
        '''
          
        
        class2print = 'ConditionProxy4Xi'
        method2print = 'sense'
        
        
        message2print = "{started" + "; sense-of-ConditionProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; ConditionProxy4Xi-sensing-by-sense-of-ConditionProxy4Xi: " + str(ConditionProxy4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                      
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-ConditionProxy4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['inheritance-of-conditionproxy2xi'] = AcquirerProxy4Xi
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-ConditionProxy4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
        
      
        message2print = "{sensed" + "; ConditionProxy4Xi-sensed-by-sense-of-ConditionProxy4Xi: " + str(ConditionProxy4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-ConditionProxy4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
         
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-ConditionProxy4Xi" usage = "reasons ConditionProxy4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons ConditionProxy4Xi class objects
        '''
                 
        
        class2print = 'ConditionProxy4Xi'
        method2print = 'reason'

        
        message2print = "{started" + "; reason-of-ConditionProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
       
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; ConditionProxy4Xi-reasoning-by-reason-of-ConditionProxy4Xi: " + str(ConditionProxy4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-ConditionProxy4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
          

        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          

                
        message2print = "{reasoned" + "; ConditionProxy4Xi-reasoned-by-reason-of-ConditionProxy4Xi: " + str(ConditionProxy4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
          
        
    
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-ConditionProxy4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
             
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-ConditionProxy4Xi" usage = "aces ConditionProxy4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces ConditionProxy4Xi class objects 
        '''
               
        
        
        class2print = 'ConditionProxy4Xi'
        method2print = 'ace'
        
        
        message2print = "{started" + "; ace-of-ConditionProxy4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
            
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; ConditionProxy4Xi-acing-by-ace-of-ConditionProxy4Xi: " + str(ConditionProxy4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                 
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-ConditionProxy4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
  


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; ConditionProxy4Xi-aced-by-ace-of-ConditionProxy4Xi: " + str(ConditionProxy4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-ConditionProxy4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>


# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

    
    

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of conditionproxy4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>



